include graphics.e
include machine.e
include mouse.e
include get.e
include font_lws.e
include kbstat.e

procedure rect(integer color, integer fill, sequence area)
    integer x1,y1,x2,y2
    x1=area[1]
    y1=area[2]
    x2=area[3]
    y2=area[4]
    polygon(color,fill,{{x1,y1},{x2,y1},{x2,y2},{x1,y2}})
end procedure

procedure draw_board(sequence color, sequence area, integer width)
    integer x1,y1,x2,y2,w
    rect(color[1],1,area)
    x1=area[1]
    y1=area[2]
    x2=area[3]
    y2=area[4]
    w=width-1
    draw_line(color[2],{{x1,y2-1},{x1,y1},{x2,y1}})
    draw_line(color[3],{{x2,y1+1},{x2,y2},{x1,y2}})
    draw_line(color[3],{{x1+w,y2-w-1},{x1+w,y1+w},{x2-w-1,y1+w}})
    draw_line(color[2],{{x2-w,y1+w},{x2-w,y2-w},{x1+w,y2-w}})
end procedure

procedure draw_button(sequence color, sequence area, integer fill, integer press)
    integer x1,y1,x2,y2
    if fill then rect(color[1],1,area) end if
    x1=area[1]
    y1=area[2]
    x2=area[3]
    y2=area[4]
    if press then
        draw_line(color[3],{{x1,y2},{x1,y1},{x2,y1}})
        draw_line(color[2],{{x2,y1+1},{x2,y2},{x1+1,y2}})
    else
        draw_line(color[2],{{x1,y2},{x1,y1},{x2,y1}})
        draw_line(color[3],{{x2,y1+1},{x2,y2},{x1+1,y2}})
    end if
end procedure

function inside(sequence area, sequence coord)
    integer x,y,x1,y1,x2,y2
    x=coord[1]
    y=coord[2]
    if atom(area[1]) then area={area} end if
    for i=1 to length(area) do
        x1=area[i][1]
        y1=area[i][2]
        x2=area[i][3]
        y2=area[i][4]
        if x>=x1 and x<=x2 and y>=y1 and y<=y2 then return i end if
    end for
    return 0
end function
----------------------------------------------------------------------------
constant gridXStart=0, gridYStart=0, gridSpace=10, gridXNum=45, gridYNum=45,
         gridColor=WHITE,
         gridXEnd=gridXStart+gridSpace*gridXNum-1,
         gridYEnd=gridYStart+gridSpace*gridYNum-1,
         gridArea={gridXStart,gridYStart,gridXEnd,gridYEnd}
constant boardColor={CYAN,BRIGHT_CYAN,GRAY},
         buttonColor={WHITE,BRIGHT_WHITE,GRAY}
constant viewXStart=475, viewYStart=5,
         viewXEnd=viewXStart+2*gridXNum-1,
         viewYEnd=viewYStart+2*gridYNum-1,
         viewArea={viewXStart,viewYStart,viewXEnd,viewYEnd}
constant palXStart=475, palYStart=315, palSpace=10, palXNum=16, palYNum=16,
         palXEnd=palXStart+palSpace*palXNum-1,
         palYEnd=palYStart+palSpace*palYNum-1,
         palArea={palXStart,palYStart,palXEnd,palYEnd}
constant menuXStart=475, menuYStart=105, menuXEnd=634, menuHeight=20,
         menuNum=10,
         menuYEnd=menuYStart+menuHeight*menuNum-1,
         MENU_NEW=1, MENU_LOAD=2, MENU_SAVE=3, MENU_MIRROR=4, MENU_MOVE=5,
         MENU_QUIT=6,
         menuText={"New","Load Bitmap","Save Bitmap","Mirror","Move",
            "Quit","Reserved","Reserved","Reserved","Reserved"}
----------------------------------------------------------------------
sequence image,savedImage,blankImage,blankGrid,fileName
    blankImage=repeat(repeat(0,gridXNum),gridYNum)
    image=blankImage
    savedImage=image
    fileName=""

function text_line(sequence text, object data)
    integer key
    mouse_pointer(0)
    poke(#41C,peek(#41A))
    fp[1..3]={1,1,1}
    rect(0,1,{0,460,450,479})
    ftc=BRIGHT_RED
    fsc=GRAY
    fx=0
    fy=460
    fprint(text)
    if sequence(data) then
        fp[4]=451-flength(text)
        data=fgets(data)
    elsif data=1 then
        while 1 do
            key=wait_key()
            if key=13 or key=27 then
                data=key
                exit
            end if
        end while
    elsif data=2 then
            key=wait_key()
    end if
    mouse_pointer(1)
    return data
end function

procedure logo()
    object junk
    junk=text_line({256+BRIGHT_BLUE}&"EUTILE  programmed  by  Euphoria 1.5a",0)
end procedure

procedure draw_a_pixel(integer color, sequence rowCol)
    integer r,c,x1,y1,x2,y2
    r=rowCol[1]
    c=rowCol[2]
    x1=gridXStart+(c-1)*gridSpace+1
    y1=gridYStart+(r-1)*gridSpace+1
    x2=x1+gridSpace-2
    y2=y1+gridSpace-2
    rect(color,1,{x1,y1,x2,y2})
    pixel({color,color},{viewXStart,viewYStart}+{c-1,r-1}*2)
    pixel({color,color},{viewXStart,viewYStart}+{c-1,r-1}*2+{0,1})
end procedure

procedure clear_grid_view()
    rect(0,1,viewArea)
    display_image({gridXStart,gridYStart},blankGrid)
end procedure

procedure menu_new()
    if compare(image,savedImage) then
        burp()
        if text_line("Not saved. ENTER to abandon or ESC to cancel",1)=13 then
            clear_grid_view()
            image=blankImage
            savedImage=image
        end if
        logo()
    elsif compare(image,blankImage) then
        clear_grid_view()
        image=blankImage
        savedImage=image
    end if
end procedure

procedure menu_load()
    integer h,w
    object name,bmpImage,junk
    if compare(image,savedImage) then
        burp()
        if text_line("Not saved. ENTER to abandon or ESC to cancel",1)=27 then
            logo()
            return
        end if
    end if
    name=text_line("File name to load (.bmp) : ","")
    if compare(name,"") then
        if not find('.',name) then name=name&".bmp" end if
        bmpImage=read_bitmap(name)
        if sequence(bmpImage) then
            bmpImage=bmpImage[2]
            h=length(bmpImage)
            w=length(bmpImage[1])
            if h<=gridYNum and w<=gridXNum then
                image=blankImage
                for i=1 to h do image[i][1..w]=bmpImage[i] end for
                clear_grid_view()
                for i=1 to h do
                    for j=1 to w do
                        if image[i][j] then
                            draw_a_pixel(image[i][j],{i,j})
                        end if
                    end for
                end for
                savedImage=image
                fileName=name
            else
                burp()
                junk=text_line("File size too big : Press any key",2)
            end if
        else
            burp()
            junk=text_line("Error in reading file : Press any key",2)
        end if
    end if
    logo()
end procedure

procedure menu_mirror()
    sequence s
    integer jr
    s=image
    image=blankImage
    clear_grid_view()
    for i=1 to gridYNum do
        for j=1 to gridXNum do
            jr=gridXNum-j+1
            if s[i][jr] then
                image[i][j]=s[i][jr]
                draw_a_pixel(image[i][j],{i,j})
            end if
        end for
    end for
end procedure
---------------------------------------
constant UP=328,DOWN=336,LEFT=331,RIGHT=333,
         C_UP=397,C_DOWN=401,C_LEFT=371,C_RIGHT=372,
         A_UP=408,A_DOWN=416,A_LEFT=411,A_RIGHT=413
function actual_region()
    integer r1,c1,r2,c2
    r1=gridYNum
    c1=gridXNum
    r2=1
    c2=1
    for i=1 to gridYNum do
        for j=1 to gridXNum do
            if image[i][j]!=0 then
                if i<r1 then r1=i end if
                if i>r2 then r2=i end if
                if j<c1 then c1=j end if
                if j>c2 then c2=j end if
            end if
        end for
    end for
    if r1>r2 then return -1 else return {r1,c1,r2,c2} end if
end function

function catch_region(sequence region, integer class)
    integer regionColor,r1,c1,r2,c2,x1,y1,x2,y2,key
    regionColor=BRIGHT_WHITE
    r1=region[1]
    c1=region[2]
    r2=region[3]
    c2=region[4]
    x1=gridXStart+(c1-1)*gridSpace
    y1=gridYStart+(r1-1)*gridSpace
    x2=gridXStart+c2*gridSpace
    y2=gridYStart+r2*gridSpace
    rect(regionColor,0,{x1,y1,x2,y2})
    poke(#41C,peek(#41A))
    while 1 do
        key=get_key()
        if key!=-1 then
            rect(gridColor,0,{x1,y1,x2,y2})
            if key=27 then
                return -1
            elsif key=13 then
                return {r1,c1,r2,c2}
            elsif class=1 then  -- move not resize
                if key=UP then
                    if r1>1 then r1=r1-1 r2=r2-1 end if
                elsif key=DOWN then
                    if r2<gridYNum then r1=r1+1 r2=r2+1 end if
                elsif key=LEFT then
                    if c1>1 then c1=c1-1 c2=c2-1 end if
                elsif key=RIGHT then
                    if c2<gridXNum then c1=c1+1 c2=c2+1 end if
                end if
            elsif class=2 then  -- resize not move
                if key=C_UP then
                    if r1>1 then r1=r1-1 end if
                elsif key=C_DOWN then
                    if r1<r2 then r1=r1+1 end if
                elsif key=C_LEFT then
                    if c1>1 then c1=c1-1 end if
                elsif key=C_RIGHT then
                    if c1<c2 then c1=c1+1 end if
                elsif key=A_UP then
                    if r2>r1 then r2=r2-1 end if
                elsif key=A_DOWN then
                    if r2<gridYNum then r2=r2+1 end if
                elsif key=A_LEFT then
                    if c2>c1 then c2=c2-1 end if
                elsif key=A_RIGHT then
                    if c2<gridXNum then c2=c2+1 end if
                end if
            end if
            x1=gridXStart+(c1-1)*gridSpace
            y1=gridYStart+(r1-1)*gridSpace
            x2=gridXStart+c2*gridSpace
            y2=gridYStart+r2*gridSpace
            rect(regionColor,0,{x1,y1,x2,y2})
        end if
    end while
end function

procedure menu_move()
    sequence s
    integer diffR,diffC
    object src,dest,junk
    mouse_pointer(0)
    src=actual_region()
    if sequence(src) then
        junk=text_line("ARROW keys to move, ENTER to accept or ESC",0)
        dest=catch_region(src,1)
        if sequence(dest) then
            diffR=dest[1]-src[1]
            diffC=dest[2]-src[2]
            s=image
            image=blankImage
            clear_grid_view()
            for i=dest[1] to dest[3] do
                for j=dest[2] to dest[4] do
                    if s[i-diffR][j-diffC] then
                        image[i][j]=s[i-diffR][j-diffC]
                        draw_a_pixel(image[i][j],{i,j})
                    end if
                end for
            end for
            if compare(s,savedImage)=0 then savedImage=image end if
        end if
        logo()
    end if
    mouse_pointer(1)
end procedure

procedure menu_save()
    sequence s,name
    integer r1,c1,r2,c2,x1,y1,x2,y2
    object region,junk
    mouse_pointer(0)
    region=actual_region()
    if sequence(region) then
        junk=text_line("Ctrl/Alt + ARROW keys, ENTER to accept or ESC",0)
        region=catch_region(region,2)
        if sequence(region) then
            r1=region[1]
            c1=region[2]
            r2=region[3]
            c2=region[4]
            x1=gridXStart+(c1-1)*gridSpace
            y1=gridYStart+(r1-1)*gridSpace
            x2=gridXStart+c2*gridSpace
            y2=gridYStart+r2*gridSpace
            rect(YELLOW,0,{x1,y1,x2,y2})
            s={}
            for i=r1 to r2 do
                s=append(s,image[i][c1..c2])
            end for
            name=fileName
            name=text_line("File name to save (.bmp) : ",name)
            if compare(name,"") then
                if not find('.',name) then name=name&".bmp" end if
                if save_bitmap({get_all_palette()*4,s},name)=BMP_SUCCESS then
                    savedImage=image
                    fileName=name
                else
                    burp()
                    junk=text_line("Error in saving to file : press any key",2)
                end if
            end if
            rect(gridColor,0,{x1,y1,x2,y2})
        end if
        logo()
    end if
    mouse_pointer(1)
end procedure

procedure menu_quit()
    object junk
    if compare(image,savedImage) then
        burp()
        if text_line("Not saved. ENTER to abandon or ESC to cancel",1)=27 then
            logo()
            return
        end if
    end if
    junk=graphics_mode(-1)
    abort(0)
end procedure
---------------------------------------------------------------------
sequence menuArea
sequence g1,g2,g3
integer x,y,x1,y1,x2,y2
integer event,move,leftDown,leftUp,rightDown,rightUp,leftDrag,
        rightDrag,c,r,color,leftColor,rightColor,menuNow,menuChoose
object junk
object mouse
--use_vesa(1)
junk=graphics_mode(257)

g1=repeat(gridColor,gridXNum*gridSpace+1)
g2=repeat(0,gridXNum*gridSpace+1)
for i=0 to gridXNum do g2[i*gridSpace+1]=gridColor end for
g3={g1}
for i=1 to gridSpace-1 do g3=g3&{g2} end for
blankGrid={}
for i=1 to gridYNum do blankGrid=blankGrid&g3 end for
blankGrid=blankGrid&{g1}

draw_board(boardColor,viewArea+{-5,-5,5,5},5)
clear_grid_view()

fselect(fload("plaza14.f"))
fp[1..3]={1,1,1}
ftc=BLACK
fsc=BRIGHT_WHITE
draw_board(boardColor,{menuXStart-5,menuYStart-5,menuXEnd+5,menuYEnd+5},5)
menuArea={}
y=menuYStart
for i=1 to menuNum do
    menuArea=append(menuArea,{menuXStart,y,menuXEnd,y+menuHeight-1})
    draw_button(buttonColor,menuArea[i],1,0)
    fy=y+3
    fcprint(menuXStart,menuXEnd,menuText[i])
    y=y+menuHeight
end for

draw_board(boardColor,palArea+{-5,-5,5,5},5)
for i=1 to palYNum do
    y1=palYStart+(i-1)*palSpace
    y2=y1+palSpace-1
    for j=1 to palXNum do
        x1=palXStart+(j-1)*palSpace
        x2=x1+palSpace-1
        rect((i-1)*palXNum+j-1,1,{x1,y1,x2,y2})
    end for
end for

logo()
-----------------------------------------------------------------------
leftDrag=0
rightDrag=0
leftColor=29
rightColor=0
menuChoose=0
set_svga_mouse()
mouse_events(MOVE+LEFT_DOWN+LEFT_UP+RIGHT_DOWN+RIGHT_UP)
mouse_pointer(1)
while 1 do
    mouse=get_mouse()
    if sequence(mouse) then
        event=mouse[1]
        x=mouse[2]
        y=mouse[3]
        move=and_bits(event,MOVE)
        leftDown=and_bits(event,LEFT_DOWN)
        leftUp=and_bits(event,LEFT_UP)
        rightDown=and_bits(event,RIGHT_DOWN)
        rightUp=and_bits(event,RIGHT_UP)
        if leftDown then leftDrag=1 end if
        if leftUp then leftDrag=0 end if
        if rightDown then rightDrag=1 end if
        if rightUp then rightDrag=0 end if
        menuNow=inside(menuArea,{x,y})
        if menuChoose and not menuNow then
            mouse_pointer(0)
            draw_button(buttonColor,menuArea[menuChoose],0,0)
            mouse_pointer(1)
            menuChoose=0
        end if

        if inside(gridArea,{x,y}) then
            if leftDrag or rightDrag then
                r=floor((y-gridYStart)/gridSpace)+1
                c=floor((x-gridXStart)/gridSpace)+1
                if kb_stat(CONTROL) and (leftDown or rightDown) then
                    color=image[r][c]
                    if leftDrag then leftColor=color
                    else rightColor=color end if
                else
                    if leftDrag then color=leftColor
                    else color=rightColor
                    end if
                    if image[r][c]!=color then
                        image[r][c]=color
                        mouse_pointer(0)
                        draw_a_pixel(color,{r,c})
                        mouse_pointer(1)
                    end if
                end if
            end if
        elsif inside(palArea,{x,y}) then
            if leftDown or rightDown then
                r=floor((y-palYStart)/palSpace)+1
                c=floor((x-palXStart)/palSpace)+1
                color=(r-1)*palXNum+c-1
                if leftDown then leftColor=color else rightColor=color end if
            end if
        elsif menuNow then
            if leftDown then
                mouse_pointer(0)
                draw_button(buttonColor,menuArea[menuNow],0,1)
                mouse_pointer(1)
                menuChoose=menuNow
            elsif menuChoose then
                if menuNow!=menuChoose then
                    mouse_pointer(0)
                    draw_button(buttonColor,menuArea[menuChoose],0,0)
                    mouse_pointer(1)
                    menuChoose=0
                elsif leftUp then
                    mouse_pointer(0)
                    draw_button(buttonColor,menuArea[menuChoose],0,0)
                    mouse_pointer(1)
if menuChoose=MENU_NEW then menu_new() end if
if menuChoose=MENU_LOAD then menu_load() end if
if menuChoose=MENU_SAVE then menu_save() end if
if menuChoose=MENU_MIRROR then menu_mirror() end if
if menuChoose=MENU_MOVE then menu_move() end if
if menuChoose=MENU_QUIT then menu_quit() end if
                    menuChoose=0
                end if
            end if
        end if

    end if
end while
junk=graphics_mode(-1)
